<?php

/**
 * Suppress "error - 0 - No summary was found for this file" on phpdoc generation
 *
 * @package WPDataAccess\Data_Dictionary
 */

namespace WPDataAccess\Data_Dictionary {

	use WPDataAccess\WPDA;

	/**
	 * Class WPDA_Dictionary_Lists
	 *
	 * @author  Peter Schulz
	 * @since   1.0.0
	 */
	class WPDA_Dictionary_Lists {

		/**
		 * List of tables for setting pages
		 *
		 * Returns an array including all tables and views in the WordPress database.
		 *
		 * Do NOT use table access control here! This list is used in the settings forms and must show ALL tables in
		 * the WordPress database.
		 *
		 * @param boolean $show_views TRUE = show views, FALSE = hide views.
		 *
		 * @return array List of database tables (and views).
		 * @since   1.0.0
		 *
		 */
		public static function get_tables( $show_views = true ) {

			global $wpdb;

			if ( false === $show_views ) {
				$and = " and table_type != 'VIEW' ";
			} else {
				$and = '';
			}

			$query = "
				select table_name,
					   create_time,
					   table_rows
				  from information_schema.tables
				 where table_schema = '$wpdb->dbname'
				 $and
				 order by table_name
			";

			return $wpdb->get_results( $query, 'ARRAY_A' ); // WPCS: unprepared SQL OK; db call ok; no-cache ok.

		}

		/**
		 * List of columns for a specific table
		 *
		 * jQuery usage: action=get_columns
		 *
		 * @return array List of column for a specific table.
		 * @since   1.6.10
		 *
		 */
		public static function get_columns() {

			if ( isset( $_REQUEST['table_name'] ) ) {
				$table_name = sanitize_text_field( wp_unslash( $_REQUEST['table_name'] ) ); // input var okay.
			} else {
				return 'ERROR_NO_TABLE';
			}

			global $wpdb;

			$query   = $wpdb->prepare( '
				  SELECT column_name
					FROM information_schema.columns 
				   WHERE table_schema = %s
					 AND table_name   = %s
				',
				[
					$wpdb->dbname,
					$table_name,
				]
			);
			$columns = $wpdb->get_results( $query, 'ARRAY_A' ); // WPCS: unprepared SQL OK; db call ok; no-cache ok.

			echo json_encode( $columns );

		}

		/**
		 * List of columns for a specific table
		 *
		 * @param $table_name
		 *
		 * @return Column in $table_name
		 * @since   2.0.10
		 *
		 */
		public static function get_table_columns( $table_name ) {

			global $wpdb;

			$query = $wpdb->prepare( '
				  SELECT column_name
					FROM information_schema.columns 
				   WHERE table_schema = %s
					 AND table_name   = %s
				',
				[
					$wpdb->dbname,
					$table_name,
				]
			);

			return $wpdb->get_results( $query, 'ARRAY_A' ); // WPCS: unprepared SQL OK; db call ok; no-cache ok.

		}

		/**
		 * List of database schemas available to user
		 *
		 * @since   1.6.0
		 */
		public function get_db_schemas() {

			global $wpdb;

			$query = '
				  SELECT schema_name
					FROM information_schema.schemata 
				   ORDER BY schema_name
			';

			return $wpdb->get_results( $query, 'ARRAY_A' ); // WPCS: unprepared SQL OK; db call ok; no-cache ok.

		}

		/**
		 * List of available engines
		 *
		 * @since   1.6.0
		 */
		public static function get_engines() {

			global $wpdb;

			$query = '
				  SELECT engine,
				         support
					FROM information_schema.engines
				   ORDER BY engine
			';

			return $wpdb->get_results( $query, 'ARRAY_A' ); // WPCS: unprepared SQL OK; db call ok; no-cache ok.

		}

		/**
		 * List of available collations
		 *
		 * @since   1.6.0
		 */
		public static function get_collations() {

			global $wpdb;

			$query = '
				SELECT character_set_name, 
				       collation_name 
				FROM   information_schema.collations
				ORDER BY character_set_name, collation_name
			';

			return $wpdb->get_results( $query, 'ARRAY_A' ); // WPCS: unprepared SQL OK; db call ok; no-cache ok.

		}

		/**
		 * Returns default collation
		 *
		 * @since   1.6.0
		 */
		public static function get_default_collation() {

			global $wpdb;

			$query = "
				SELECT default_character_set_name,
				       default_collation_name
				FROM   information_schema.schemata
				WHERE  schema_name = '$wpdb->dbname'
				GROUP BY schema_name
			";

			return $wpdb->get_results( $query, 'ARRAY_A' ); // WPCS: unprepared SQL OK; db call ok; no-cache ok.

		}

	}

}
