<?php

/**
 * Suppress "error - 0 - No summary was found for this file" on phpdoc generation
 *
 * @package WPDataProjects\Simple_Form
 */

namespace WPDataProjects\Simple_Form {

	use WPDataAccess\Simple_Form\WPDA_Simple_Form;
	use WPDataAccess\Simple_Form\WPDA_Simple_Form_Item_Enum;
	use WPDataAccess\Simple_Form\WPDA_Simple_Form_Item_Image;
	use WPDataAccess\Simple_Form\WPDA_Simple_Form_Item_Media;
	use WPDataProjects\Project\WPDP_Project_Design_Table_Model;

	/**
	 * Class WPDP_Simple_Form extends WPDA_Simple_Form
	 *
	 * Uses table options to hide items and add lookups to data entry form
	 *
	 * @see WPDA_Simple_Form
	 *
	 * @author  Peter Schulz
	 * @since   2.0.0
	 */
	class WPDP_Simple_Form extends WPDA_Simple_Form {

		/**
		 * WPDP_Simple_Form constructor.
		 *
		 * @param       $schema_name
		 * @param       $table_name
		 * @param       $wpda_list_columns
		 * @param array $args
		 */
		public function __construct( $schema_name, $table_name, &$wpda_list_columns, $args = [] ) {
			if ( isset ( $args['title'] ) ) {
				$this->title = $args['title'];
			}

			parent::__construct( $schema_name, $table_name, $wpda_list_columns, $args );
		}

		/**
		 * Overwrites method prepare_items
		 *
		 * Uses table options to hide items and add lookups to data entry form
		 *
		 * @param bool $set_back_form_values
		 */
		protected function prepare_items( $set_back_form_values = false ) {
			parent::prepare_items( $set_back_form_values );

			foreach ( $this->wpda_list_columns->get_table_columns() as $columns ) {
				// Hide columns which have show attribute disabled.
				if ( isset ( $columns['show'] ) && ! $columns['show'] ) {
					$item_index = $this->get_item_index( $columns['column_name'] );
					if ( isset( $this->form_items[ $item_index ] ) ) {
						$this->form_items[ $item_index ]->set_hide_item( true );
					}
				}

				// Set default value if available
				if ( isset ( $columns['default'] ) && '' !== $columns['default'] ) {
					$item_default_value = $columns['default'];
					if ( '$$USER$$' === $item_default_value ) {
						$wp_user            = wp_get_current_user();
						$item_default_value = $wp_user->data->user_login;
					}
					$item_index = $this->get_item_index( $columns['column_name'] );
					if ( isset( $this->form_items[ $item_index ] ) ) {
						$this->form_items[ $item_index ]->set_item_default_value( $item_default_value );
					}
				}
			}

			// Check if there are any lookup items defined for this table.
			$lookup_column_name = [];
			$tableform          = WPDP_Project_Design_Table_Model::get_column_options( $this->table_name, 'tableform' );

			$i = 0;
			if ( null !== $tableform ) {
				foreach ( $tableform as $tableform_item ) {
					// Process lookup items
					if ( isset( $tableform_item->lookup ) && false !== $tableform_item->lookup ) {
						$lookup_column_name[ $tableform_item->column_name ] = $tableform_item->lookup;
					}

					if ( is_admin() ) {
						if ( isset( $tableform_item->item_type ) ) {
							// Process images
							if ( 'image' === $tableform_item->item_type ) {
								$class_path = explode('\\', get_class($this->form_items[ $i ]));
								$class_name = array_pop($class_path);
								if ( 'WPDA_Simple_Form_Item_Image' !== $class_name ) {
									$this->form_items[ $i ] = new WPDA_Simple_Form_Item_Image( $this->form_items[ $i ] );
								}
							}

							// Process attachmemts
							if ( 'attachment' === $tableform_item->item_type ) {
								$class_path = explode('\\', get_class($this->form_items[ $i ]));
								$class_name = array_pop($class_path);
								if ( 'WPDA_Simple_Form_Item_Media' !== $class_name ) {
									$this->form_items[ $i ] = new WPDA_Simple_Form_Item_Media( $this->form_items[ $i ] );
								}
							}
						}
					}

					$i++;
				}
			}
			if ( sizeof( $lookup_column_name ) > 0 ) {
				// Process lookup items and create listboxes.
				$lookups       = [];
				$relationships = WPDP_Project_Design_Table_Model::get_column_options( $this->table_name, 'relationships' );

				if ( null !== $relationships ) {
					if ( isset( $relationships['relationships'] ) ) {
						foreach ( $relationships['relationships'] as $relationship ) {
							if ( isset( $relationship->relation_type ) && 'lookup' === $relationship->relation_type ) {
								array_push( $lookups, $relationship );
							}
						}
					}
				}

				$i = 0;
				foreach ( $this->form_items as $item ) {
					if ( isset( $lookup_column_name[ $item->get_item_name() ] ) ) {
						foreach ( $lookups as $lookup ) {
							// Lookups are always based on a single column. Use first element of array.
							$source_column_name = $lookup->source_column_name[0];
							if ( $source_column_name === $item->get_item_name() ) {
								$target_column_name = $lookup->target_column_name[0];
								$target_table_name  = $lookup->target_table_name;

								if ( isset( $lookup_column_name[ $source_column_name ] ) ) {
									$lookup_sql =
										"select `{$lookup_column_name[ $source_column_name ]}`, `$target_column_name` " .
										"from `$target_table_name` " .
										"order by `{$lookup_column_name[ $source_column_name ]}`, `$target_column_name`";

									global $wpdb;
									$rows = $wpdb->get_results( $lookup_sql, 'ARRAY_A' );

									$lov_values  = [];
									$lov_options = [];

									if ( isset( $relationships['table'] ) ) {
										foreach ( $relationships['table'] as $table_column ) {
											if ( isset( $table_column->column_name ) && isset( $table_column->mandatory ) ) {
												if ( $table_column->column_name === $source_column_name && 'No' === $table_column->mandatory ) {
													array_push( $lov_values, '' );
													array_push( $lov_options, '' );
												}
											}
										}
									}

									foreach ( $rows as $row ) {
										$lov_value = $row[ $lookup_column_name[ $source_column_name ] ] . ' (' . $row[ $target_column_name ] . ')';
										array_push( $lov_values, $lov_value );
										array_push( $lov_options, $row[ $target_column_name ] );
									}

									$item->set_enum( $lov_values );
									$item->set_enum_options( $lov_options );
									$this->form_items[ $i ] = new WPDA_Simple_Form_Item_Enum( $item );
								}
							}
						}
					}
					$i++;
				}
			}

		}

	}

}